@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2004-2005 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2004-2006 William Jon McCann <mccann@jhu.edu>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -36,66 +36,10 @@
 
 #include "gs-lock-plug.h"
 
-#include "passwd.h"
+#include "gs-auth.h"
 #include "setuid.h"
 
-
-/* Profiling stuff adapted from gtkfilechooserdefault */
-/* To use run:
- *  strace -ttt -f -o logfile.txt ./gnome-screensaver-dialog
- */
-
-#undef PROFILE_LOCK_DIALOG
-#ifdef PROFILE_LOCK_DIALOG
-
-#define PROFILE_INDENT 4
-static int profile_indent;
-
-static void
-profile_add_indent (int indent)
-{
-        profile_indent += indent;
-        if (profile_indent < 0) {
-                g_error ("You screwed up your indentation");
-        }
-}
-
-static void
-_gs_lock_plug_profile_log (const char *func,
-                           int         indent,
-                           const char *msg1,
-                           const char *msg2)
-{
-        char *str;
-
-        if (indent < 0) {
-                profile_add_indent (indent);
-        }
-
-        if (profile_indent == 0) {
-                str = g_strdup_printf ("MARK: %s: %s %s %s", G_STRLOC, func, msg1 ? msg1 : "", msg2 ? msg2 : "");
-        } else {
-                str = g_strdup_printf ("MARK: %s: %*c %s %s %s", G_STRLOC, profile_indent - 1, ' ', func, msg1 ? msg1 : "", msg2 ? msg2 : "");
-        }
-
-        access (str, F_OK);
-
-        g_free (str);
-
-        if (indent > 0) {
-                profile_add_indent (indent);
-        }
-}
-
-#define profile_start(x, y) _gs_lock_plug_profile_log (G_STRFUNC, PROFILE_INDENT, x, y)
-#define profile_end(x, y)   _gs_lock_plug_profile_log (G_STRFUNC, -PROFILE_INDENT, x, y)
-#define profile_msg(x, y)   _gs_lock_plug_profile_log (NULL, 0, x, y)
-#else
-#define profile_start(x, y)
-#define profile_end(x, y)
-#define profile_msg(x, y)
-#endif
-
+#include "gs-debug.h"
 
 static gboolean verbose        = FALSE;
 static gboolean show_version   = FALSE;
@@ -135,13 +79,13 @@ print_id (GtkWidget *widget)
 {
         char *id;
 
-        profile_start ("start", NULL);
+        gs_profile_start (NULL);
 
         id = get_id_string (widget);
         printf ("WINDOW ID=%s\n", id);
         fflush (stdout);
 
-        profile_end ("end", NULL);
+        gs_profile_end (NULL);
 
         g_free (id);
 
@@ -162,6 +106,22 @@ response_ok (void)
         fflush (stdout);
 }
 
+static gboolean
+quit_response_ok (void)
+{
+        response_ok ();
+        gtk_main_quit ();
+        return FALSE;
+}
+
+static gboolean
+quit_response_cancel (void)
+{
+        response_cancel ();
+        gtk_main_quit ();
+        return FALSE;
+}
+
 static void
 response_lock_init_failed (void)
 {
@@ -169,21 +129,192 @@ response_lock_init_failed (void)
         response_ok ();
 }
 
+static char *
+request_response (GSLockPlug *plug,
+                  const char *prompt,
+                  gboolean    visible)
+{
+        int   response;
+        char *text;
+
+        gs_lock_plug_show_prompt (plug, prompt, visible);
+        response = gs_lock_plug_run (plug);
+
+        gs_debug ("got response: %d", response);
+
+        text = NULL;
+        if (response == GS_LOCK_PLUG_RESPONSE_OK) {
+                gs_lock_plug_get_text (plug, &text);
+        }
+
+        return text;
+}
+
+/* Adapted from GDM2 daemon/verify-pam.c on 2006-06-13 */
+static const char *
+maybe_translate_message (const char *msg)
+{
+	char              *s;
+	const char        *ret;
+	static GHashTable *hash = NULL;
+
+	if (hash == NULL) {
+		/* Here we come with some fairly standard messages so that
+		   we have as much as possible translated.  Should really be
+		   translated in pam I suppose.  This way we can "change"
+		   some of these messages to be more sane. */
+		hash = g_hash_table_new (g_str_hash, g_str_equal);
+		/* login: is whacked always translate to Username: */
+		g_hash_table_insert (hash, "login:", _("Username:"));
+		g_hash_table_insert (hash, "Username:", _("Username:"));
+		g_hash_table_insert (hash, "username:", _("Username:"));
+		g_hash_table_insert (hash, "Password:", _("Password:"));
+		g_hash_table_insert (hash, "password:", _("Password:"));
+		g_hash_table_insert (hash, "You are required to change your password immediately (password aged)", _("You are required to change your password immediately (password aged)"));
+		g_hash_table_insert (hash, "You are required to change your password immediately (root enforced)", _("You are required to change your password immediately (root enforced)"));
+		g_hash_table_insert (hash, "Your account has expired; please contact your system administrator", _("Your account has expired; please contact your system administrator"));
+		g_hash_table_insert (hash, "No password supplied", _("No password supplied"));
+		g_hash_table_insert (hash, "Password unchanged", _("Password unchanged"));
+		g_hash_table_insert (hash, "Can not get username", _("Can not get username"));
+		g_hash_table_insert (hash, "Retype new UNIX password:", _("Retype new UNIX password:"));
+		g_hash_table_insert (hash, "Enter new UNIX password:", _("Enter new UNIX password:"));
+		g_hash_table_insert (hash, "(current) UNIX password:", _("(current) UNIX password:"));
+		g_hash_table_insert (hash, "Error while changing NIS password.", _("Error while changing NIS password."));
+		g_hash_table_insert (hash, "You must choose a longer password", _("You must choose a longer password"));
+		g_hash_table_insert (hash, "Password has been already used. Choose another.", _("Password has been already used. Choose another."));
+		g_hash_table_insert (hash, "You must wait longer to change your password", _("You must wait longer to change your password"));
+		g_hash_table_insert (hash, "Sorry, passwords do not match", _("Sorry, passwords do not match"));
+		/* FIXME: what about messages which have some variables in them, perhaps try to do those as well */
+	}
+
+	s = g_strstrip (g_strdup (msg));
+	ret = g_hash_table_lookup (hash, s);
+	g_free (s);
+
+	if (ret != NULL) {
+		return ret;
+        } else {
+		return msg;
+        }
+}
+
+static gboolean
+auth_message_handler (GSAuthMessageStyle style,
+                      const char        *msg,
+                      char             **response,
+                      gpointer           data)
+{
+        gboolean    ret;
+        GSLockPlug *plug;
+	const char *message;
+
+        plug = GS_LOCK_PLUG (data);
+
+        gs_profile_start (NULL);
+        gs_debug ("Got message style %d: '%s'", style, msg);
+
+        ret = TRUE;
+        *response = NULL;
+	message = maybe_translate_message (msg);
+
+        switch (style) {
+        case GS_AUTH_MESSAGE_PROMPT_ECHO_ON:
+                if (msg != NULL) {
+                        char *resp;
+                        resp = request_response (plug, message, TRUE);
+                        *response = resp;
+                }
+                break;
+        case GS_AUTH_MESSAGE_PROMPT_ECHO_OFF:
+                if (msg != NULL) {
+                        char *resp;
+                        resp = request_response (plug, message, FALSE);
+                        *response = resp;
+                }
+                break;
+        case GS_AUTH_MESSAGE_ERROR_MSG:
+                gs_lock_plug_show_message (plug, message);
+                break;
+        case GS_AUTH_MESSAGE_TEXT_INFO:
+                gs_lock_plug_show_message (plug, message);
+                break;
+        default:
+                g_assert_not_reached ();
+        }
+
+        if (*response == NULL) {
+                gs_debug ("Got not response");
+                ret = FALSE;
+        }
+
+        gs_profile_end (NULL);
+
+        return ret;
+}
+
+static gboolean
+reset_idle_cb (GSLockPlug *plug)
+{
+        gtk_widget_set_sensitive (GTK_WIDGET (plug), TRUE);
+        gs_lock_plug_show_message (plug, NULL);
+
+        return FALSE;
+}
+
+static gboolean
+do_auth_check (GSLockPlug *plug)
+{
+        GError *error;
+        gboolean res;
+
+        error = NULL;
+
+        res = gs_auth_verify_user (g_get_user_name (), g_getenv ("DISPLAY"), auth_message_handler, plug, &error);
+
+        gs_debug ("Verify user returned: %s", res ? "TRUE" : "FALSE");
+        if (! res) {
+                if (error != NULL) {
+                        gs_debug ("Verify user returned error: %s", error->message);
+                        gs_lock_plug_show_message (plug, error->message);
+                } else {
+                        gs_lock_plug_show_message (plug, _("Authentication failed."));
+                }
+
+                gtk_widget_set_sensitive (GTK_WIDGET (plug), FALSE);
+                g_timeout_add (3000, (GSourceFunc)reset_idle_cb, plug);
+
+                printf ("NOTICE=AUTH FAILED\n");
+                fflush (stdout);
+
+                if (error != NULL) {
+                        g_error_free (error);
+                }
+        }
+
+        return res;
+}
+
 static void
 response_cb (GSLockPlug *plug,
              gint        response_id)
 {
-        switch (response_id) {
-        case (GS_LOCK_PLUG_RESPONSE_OK):
-                response_ok ();
-                break;
-        case (GS_LOCK_PLUG_RESPONSE_CANCEL):
-        default:
-                response_cancel ();
-                break;
+        if (response_id == GS_LOCK_PLUG_RESPONSE_CANCEL) {
+                quit_response_cancel ();
+        }
+}
+
+static gboolean
+auth_check_idle (GSLockPlug *plug)
+{
+        gboolean res;
+
+        res = do_auth_check (plug);
+
+        if (res) {
+                g_idle_add ((GSourceFunc)quit_response_ok, NULL);
         }
 
-        gtk_main_quit ();
+        return !res;
 }
 
 static gboolean
@@ -191,7 +322,7 @@ popup_dialog_idle (void)
 {
         GtkWidget *widget;
 
-        profile_start ("start", NULL);
+        gs_profile_start (NULL);
 
         widget = gs_lock_plug_new ();
 
@@ -213,7 +344,9 @@ popup_dialog_idle (void)
 
         print_id (widget);
 
-        profile_end ("end", NULL);
+        g_idle_add ((GSourceFunc)auth_check_idle, widget);
+
+        gs_profile_end (NULL);
 
         return FALSE;
 }
@@ -237,11 +370,11 @@ privileged_initialization (int     *argc,
         char    *orig_uid;
         char    *uid_message;
 
-        profile_start ("start", NULL);
+        gs_profile_start (NULL);
 
 #ifndef NO_LOCKING
         /* before hack_uid () for proper permissions */
-        lock_priv_init (*argc, argv, verbose);
+        gs_auth_priv_init ();
 #endif /* NO_LOCKING */
 
         ret = hack_uid (&nolock_reason,
@@ -260,7 +393,7 @@ privileged_initialization (int     *argc,
         g_free (orig_uid);
         g_free (uid_message);
 
-        profile_end ("end", NULL);
+        gs_profile_end (NULL);
 
         return ret;
 }
@@ -278,7 +411,7 @@ lock_initialization (int     *argc,
                      char   **nolock_reason,
                      gboolean verbose)
 {
-        profile_start ("start", NULL);
+        gs_profile_start (NULL);
 
         if (nolock_reason) {
                 *nolock_reason = NULL;
@@ -293,7 +426,7 @@ lock_initialization (int     *argc,
 #else /* !NO_LOCKING */
 
         /* Finish initializing locking, now that we're out of privileged code. */
-        if (! lock_init (*argc, argv, verbose)) {
+        if (! gs_auth_init ()) {
                 if (nolock_reason) {
                         *nolock_reason = g_strdup ("error getting password");
                 }
@@ -340,7 +473,7 @@ lock_initialization (int     *argc,
 
 #endif /* NO_LOCKING */
 
-        profile_end ("end", NULL);
+        gs_profile_end (NULL);
 
         return TRUE;
 }
@@ -356,9 +489,9 @@ main (int    argc,
         bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 # ifdef HAVE_BIND_TEXTDOMAIN_CODESET
         bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-# endif 
+# endif
         textdomain (GETTEXT_PACKAGE);
-#endif 
+#endif
 
         g_type_init ();
 
@@ -368,7 +501,7 @@ main (int    argc,
                 exit (1);
         }
 
-        profile_start ("start", NULL);
+        gs_profile_start (NULL);
 
         if (! privileged_initialization (&argc, argv, verbose)) {
                 response_lock_init_failed ();
@@ -398,11 +531,14 @@ main (int    argc,
                 exit (1);
         }
 
+        gs_debug_init (verbose, FALSE);
+
         g_idle_add ((GSourceFunc)popup_dialog_idle, NULL);
 
         gtk_main ();
 
-        profile_end ("end", NULL);
+        gs_profile_end (NULL);
+        gs_debug_shutdown ();
 
 	return 0;
 }
