@@ -49,7 +49,7 @@ static gpointer grab_object = NULL;
 
 struct GSGrabPrivate
 {
-        gboolean   mouse_hide_cursor;
+        guint      mouse_hide_cursor : 1;
         GdkWindow *mouse_grab_window;
         GdkWindow *keyboard_grab_window;
         GdkScreen *mouse_grab_screen;
@@ -392,6 +392,9 @@ gs_grab_release (GSGrab *grab)
 
         /* FIXME: is it right to enable this ? */
         xorg_lock_smasher_set_active (grab, TRUE);
+
+        gdk_display_sync (gdk_display_get_default ());
+        gdk_flush ();
 }
 
 gboolean
