@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2004-2005 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2004-2006 William Jon McCann <mccann@jhu.edu>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -72,7 +72,6 @@ struct GSListenerPrivate
         guint           throttled : 1;
         GHashTable     *inhibitors;
         GHashTable     *throttlers;
-
         time_t          active_start;
         time_t          session_idle_start;
 };
@@ -90,7 +89,7 @@ enum {
         LOCK,
         CYCLE,
         QUIT,
-        POKE,
+        SIMULATE_USER_ACTIVITY,
         ACTIVE_CHANGED,
         THROTTLE_CHANGED,
         LAST_SIGNAL
@@ -116,7 +115,6 @@ gs_listener_vtable = { &gs_listener_unregister_handler,
                        NULL,
                        NULL };
 
-static GObjectClass *parent_class = NULL;
 static guint         signals [LAST_SIGNAL] = { 0, };
 
 G_DEFINE_TYPE (GSListener, gs_listener, G_TYPE_OBJECT)
@@ -125,8 +123,9 @@ GQuark
 gs_listener_error_quark (void)
 {
         static GQuark quark = 0;
-        if (!quark)
+        if (!quark) {
                 quark = g_quark_from_static_string ("gs_listener_error");
+        }
 
         return quark;
 }
@@ -216,6 +215,9 @@ gs_listener_send_signal_active_changed (GSListener *listener)
 
         g_return_if_fail (listener != NULL);
 
+        gs_debug ("Sending the ActiveChanged(%s) signal on the session bus",
+                  listener->priv->active ? "TRUE" : "FALSE");
+
         message = dbus_message_new_signal (GS_LISTENER_PATH,
                                            GS_LISTENER_SERVICE,
                                            "ActiveChanged");
@@ -1080,17 +1082,112 @@ listener_get_session_idle_time (GSListener     *listener,
         return DBUS_HANDLER_RESULT_HANDLED;
 }
 
+static DBusHandlerResult
+do_introspect (DBusConnection *connection,
+               DBusMessage    *message,
+               dbus_bool_t     local_interface)
+{
+        DBusMessage *reply;
+        GString     *xml;
+        char        *xml_string;
+
+        /* standard header */
+        xml = g_string_new ("<!DOCTYPE node PUBLIC \"-//freedesktop//DTD D-BUS Object Introspection 1.0//EN\"\n"
+                            "\"http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd\">\n"
+                            "<node>\n"
+                            "  <interface name=\"org.freedesktop.DBus.Introspectable\">\n"
+                            "    <method name=\"Introspect\">\n"
+                            "      <arg name=\"data\" direction=\"out\" type=\"s\"/>\n"
+                            "    </method>\n"
+                            "  </interface>\n");
+
+        /* ScreenSaver interface */
+        xml = g_string_append (xml,
+                               "  <interface name=\"org.gnome.ScreenSaver\">\n"
+                               "    <method name=\"Lock\">\n"
+                               "    </method>\n"
+                               "    <method name=\"Cycle\">\n"
+                               "    </method>\n"
+                               "    <method name=\"SimulateUserActivity\">\n"
+                               "    </method>\n"
+                               "    <method name=\"Inhibit\">\n"
+                               "      <arg name=\"application_name\" direction=\"in\" type=\"s\"/>\n"
+                               "      <arg name=\"reason\" direction=\"in\" type=\"s\"/>\n"
+                               "      <arg name=\"cookie\" direction=\"out\" type=\"u\"/>\n"
+                               "    </method>\n"
+                               "    <method name=\"UnInhibit\">\n"
+                               "      <arg name=\"cookie\" direction=\"in\" type=\"u\"/>\n"
+                               "    </method>\n"
+                               "    <method name=\"Throttle\">\n"
+                               "      <arg name=\"application_name\" direction=\"in\" type=\"s\"/>\n"
+                               "      <arg name=\"reason\" direction=\"in\" type=\"s\"/>\n"
+                               "      <arg name=\"cookie\" direction=\"out\" type=\"u\"/>\n"
+                               "    </method>\n"
+                               "    <method name=\"UnThrottle\">\n"
+                               "      <arg name=\"cookie\" direction=\"in\" type=\"u\"/>\n"
+                               "    </method>\n"
+                               "    <method name=\"GetActive\">\n"
+                               "      <arg name=\"value\" direction=\"out\" type=\"b\"/>\n"
+                               "    </method>\n"
+                               "    <method name=\"GetActiveTime\">\n"
+                               "      <arg name=\"seconds\" direction=\"out\" type=\"u\"/>\n"
+                               "    </method>\n"
+                               "    <method name=\"SetActive\">\n"
+                               "      <arg name=\"value\" direction=\"in\" type=\"b\"/>\n"
+                               "    </method>\n"
+                               "    <method name=\"GetSessionIdle\">\n"
+                               "      <arg name=\"value\" direction=\"out\" type=\"b\"/>\n"
+                               "    </method>\n"
+                               "    <method name=\"GetSessionIdleTime\">\n"
+                               "      <arg name=\"seconds\" direction=\"out\" type=\"u\"/>\n"
+                               "    </method>\n"
+                               "    <signal name=\"ActiveChanged\">\n"
+                               "      <arg name=\"new_value\" type=\"b\"/>\n"
+                               "    </signal>\n"
+                               "    <signal name=\"SessionIdleChanged\">\n"
+                               "      <arg name=\"new_value\" type=\"b\"/>\n"
+                               "    </signal>\n"
+                               "    <signal name=\"AuthenticationRequestBegin\">\n"
+                               "    </signal>\n"
+                               "    <signal name=\"AuthenticationRequestEnd\">\n"
+                               "    </signal>\n"
+                               "  </interface>\n");
+
+        reply = dbus_message_new_method_return (message);
+
+        xml = g_string_append (xml, "</node>\n");
+        xml_string = g_string_free (xml, FALSE);
+
+        dbus_message_append_args (reply,
+                                  DBUS_TYPE_STRING, &xml_string,
+                                  DBUS_TYPE_INVALID);
+
+        g_free (xml_string);
+
+        if (reply == NULL) {
+                g_error ("No memory");
+        }
+
+        if (! dbus_connection_send (connection, reply, NULL)) {
+                g_error ("No memory");
+        }
+
+        dbus_message_unref (reply);
+
+        return DBUS_HANDLER_RESULT_HANDLED;
+}
+
 static DBusHandlerResult
 listener_dbus_filter_handle_methods (DBusConnection *connection,
-                                     DBusMessage    *message, 
+                                     DBusMessage    *message,
                                      void           *user_data,
                                      dbus_bool_t     local_interface)
 {
         GSListener *listener = GS_LISTENER (user_data);
 
 #if 0
-        g_message ("obj_path=%s interface=%s method=%s destination=%s", 
-                   dbus_message_get_path (message), 
+        g_message ("obj_path=%s interface=%s method=%s destination=%s",
+                   dbus_message_get_path (message),
                    dbus_message_get_interface (message),
                    dbus_message_get_member (message),
                    dbus_message_get_destination (message));
@@ -1123,25 +1220,28 @@ listener_dbus_filter_handle_methods (DBusConnection *connection,
         if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "UnThrottle")) {
                 return listener_dbus_remove_ref_entry (listener, REF_ENTRY_TYPE_THROTTLE, connection, message);
         }
-        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "setActive")) {
+        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "SetActive")) {
                 return listener_set_property (listener, connection, message, PROP_ACTIVE);
         }
-        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "getActive")) {
+        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "GetActive")) {
                 return listener_get_property (listener, connection, message, PROP_ACTIVE);
         }
-        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "getActiveTime")) {
+        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "GetActiveTime")) {
                 return listener_get_active_time (listener, connection, message);
         }
-        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "getSessionIdle")) {
+        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "GetSessionIdle")) {
                 return listener_get_property (listener, connection, message, PROP_SESSION_IDLE);
         }
-        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "getSessionIdleTime")) {
+        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "GetSessionIdleTime")) {
                 return listener_get_session_idle_time (listener, connection, message);
         }
-        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "Poke")) {
-                g_signal_emit (listener, signals [POKE], 0);
+        if (dbus_message_is_method_call (message, GS_LISTENER_SERVICE, "SimulateUserActivity")) {
+                g_signal_emit (listener, signals [SIMULATE_USER_ACTIVITY], 0);
                 return DBUS_HANDLER_RESULT_HANDLED;
         }
+	if (dbus_message_is_method_call (message, "org.freedesktop.DBus.Introspectable", "Introspect")) {
+		return do_introspect (connection, message, local_interface);
+        }
 
         return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 }
@@ -1155,8 +1255,8 @@ gs_listener_message_handler (DBusConnection *connection,
         g_return_val_if_fail (message != NULL, DBUS_HANDLER_RESULT_NOT_YET_HANDLED);
 
 #if 0
-        g_message ("obj_path=%s interface=%s method=%s destination=%s", 
-                   dbus_message_get_path (message), 
+        g_message ("obj_path=%s interface=%s method=%s destination=%s",
+                   dbus_message_get_path (message),
                    dbus_message_get_interface (message),
                    dbus_message_get_member (message),
                    dbus_message_get_destination (message));
@@ -1180,7 +1280,6 @@ gs_listener_message_handler (DBusConnection *connection,
                 return DBUS_HANDLER_RESULT_HANDLED;
         } else if (dbus_message_is_signal (message, DBUS_INTERFACE_LOCAL, "Disconnected") &&
                    strcmp (dbus_message_get_path (message), DBUS_PATH_LOCAL) == 0) {
-                
                 dbus_connection_unref (connection);
 
                 return DBUS_HANDLER_RESULT_HANDLED;
@@ -1242,8 +1341,8 @@ listener_dbus_filter_function (DBusConnection *connection,
         path = dbus_message_get_path (message);
 
         /*
-        g_message ("obj_path=%s interface=%s method=%s", 
-                   dbus_message_get_path (message), 
+        g_message ("obj_path=%s interface=%s method=%s",
+                   dbus_message_get_path (message),
                    dbus_message_get_interface (message),
                    dbus_message_get_member (message));
         */
@@ -1328,8 +1427,6 @@ gs_listener_class_init (GSListenerClass *klass)
 {
         GObjectClass   *object_class = G_OBJECT_CLASS (klass);
 
-        parent_class = g_type_class_peek_parent (klass);
-
         object_class->finalize     = gs_listener_finalize;
         object_class->get_property = gs_listener_get_property;
         object_class->set_property = gs_listener_set_property;
@@ -1364,11 +1461,11 @@ gs_listener_class_init (GSListenerClass *klass)
                               g_cclosure_marshal_VOID__VOID,
                               G_TYPE_NONE,
                               0);
-        signals [POKE] =
-                g_signal_new ("poke",
+        signals [SIMULATE_USER_ACTIVITY] =
+                g_signal_new ("simulate-user-activity",
                               G_TYPE_FROM_CLASS (object_class),
                               G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GSListenerClass, poke),
+                              G_STRUCT_OFFSET (GSListenerClass, simulate_user_activity),
                               NULL,
                               NULL,
                               g_cclosure_marshal_VOID__VOID,
@@ -1421,7 +1518,7 @@ screensaver_is_running (DBusConnection *connection)
 {
         DBusError error;
         gboolean  exists;
-              
+
         g_return_val_if_fail (connection != NULL, FALSE);
 
         dbus_error_init (&error);
@@ -1521,7 +1618,6 @@ gs_listener_init (GSListener *listener)
                                                             g_int_equal,
                                                             NULL,
                                                             (GDestroyNotify)gs_listener_ref_entry_free);
-
 }
 
 static void
@@ -1544,7 +1640,7 @@ gs_listener_finalize (GObject *object)
                 g_hash_table_destroy (listener->priv->throttlers);
         }
 
-        G_OBJECT_CLASS (parent_class)->finalize (object);
+        G_OBJECT_CLASS (gs_listener_parent_class)->finalize (object);
 }
 
 GSListener *
