@@ -99,7 +99,6 @@ enum {
 
 #define FADE_TIMEOUT 1000
 
-static GObjectClass *parent_class = NULL;
 static guint         signals [LAST_SIGNAL] = { 0, };
 
 G_DEFINE_TYPE (GSManager, gs_manager, G_TYPE_OBJECT)
@@ -238,6 +237,15 @@ gs_manager_set_mode (GSManager  *manager,
         manager->priv->saver_mode = mode;
 }
 
+static void
+free_themes (GSManager *manager)
+{
+        if (manager->priv->themes) {
+                g_slist_foreach (manager->priv->themes, (GFunc)g_free, NULL);
+                g_slist_free (manager->priv->themes);
+        }
+}
+
 void
 gs_manager_set_themes (GSManager *manager,
                        GSList    *themes)
@@ -246,10 +254,7 @@ gs_manager_set_themes (GSManager *manager,
 
         g_return_if_fail (GS_IS_MANAGER (manager));
 
-        if (manager->priv->themes) {
-                g_slist_foreach (manager->priv->themes, (GFunc)g_free, NULL);
-                g_slist_free (manager->priv->themes);
-        }
+        free_themes (manager);
         manager->priv->themes = NULL;
 
         for (l = themes; l; l = l->next) {
@@ -612,8 +617,6 @@ gs_manager_class_init (GSManagerClass *klass)
 {
         GObjectClass   *object_class = G_OBJECT_CLASS (klass);
 
-        parent_class = g_type_class_peek_parent (klass);
-
         object_class->finalize     = gs_manager_finalize;
         object_class->get_property = gs_manager_get_property;
         object_class->set_property = gs_manager_set_property;
@@ -749,6 +752,15 @@ remove_timers (GSManager *manager)
         remove_cycle_timer (manager);
 }
 
+static void
+remove_unfade_idle (GSManager *manager)
+{
+        if (manager->priv->unfade_idle_id > 0) {
+                g_source_remove (manager->priv->unfade_idle_id);
+                manager->priv->unfade_idle_id = 0;
+        }
+}
+
 static void
 gs_manager_finalize (GObject *object)
 {
@@ -762,8 +774,10 @@ gs_manager_finalize (GObject *object)
 
         g_return_if_fail (manager->priv != NULL);
 
+        free_themes (manager);
         g_free (manager->priv->logout_command);
 
+        remove_unfade_idle (manager);
         remove_timers (manager);
 
         gs_grab_release (manager->priv->grab);
@@ -783,7 +797,7 @@ gs_manager_finalize (GObject *object)
         g_object_unref (manager->priv->fade);
         g_object_unref (manager->priv->grab);
 
-        G_OBJECT_CLASS (parent_class)->finalize (object);
+        G_OBJECT_CLASS (gs_manager_parent_class)->finalize (object);
 }
 
 static gboolean
@@ -922,6 +936,7 @@ manager_maybe_grab_window (GSManager *manager,
         grabbed = FALSE;
         if (gs_window_get_screen (window) == screen
             && gs_window_get_monitor (window) == monitor) {
+                gs_debug ("Moving grab to %p", window);
                 gs_grab_move_to_window (manager->priv->grab,
                                         gs_window_get_gdk_window (window),
                                         gs_window_get_screen (window),
@@ -955,15 +970,6 @@ unfade_idle (GSManager *manager)
 }
 
 
-static void
-remove_unfade_idle (GSManager *manager)
-{
-        if (manager->priv->unfade_idle_id > 0) {
-                g_source_remove (manager->priv->unfade_idle_id);
-                manager->priv->unfade_idle_id = 0;
-        }
-}
-
 static void
 add_unfade_idle (GSManager *manager)
 {
@@ -1207,7 +1213,7 @@ gs_manager_activate (GSManager *manager)
                 return FALSE;
         }
 
-        if (! manager->priv->windows) {
+        if (manager->priv->windows == NULL) {
                 gs_manager_create (GS_MANAGER (manager));
         }
 
@@ -1251,6 +1257,7 @@ gs_manager_deactivate (GSManager *manager)
                 return FALSE;
         }
 
+        remove_unfade_idle (manager);
         remove_timers (manager);
 
         gs_grab_release (manager->priv->grab);
