@@ -70,7 +70,6 @@ static GConfEnumStringPair mode_enum_map [] = {
        { 0, NULL }
 };
 
-static GObjectClass *parent_class = NULL;
 static guint         signals [LAST_SIGNAL] = { 0, };
 
 G_DEFINE_TYPE (GSPrefs, gs_prefs, G_TYPE_OBJECT)
@@ -106,8 +105,6 @@ gs_prefs_class_init (GSPrefsClass *klass)
 {
         GObjectClass   *object_class = G_OBJECT_CLASS (klass);
 
-        parent_class = g_type_class_peek_parent (klass);
-
         object_class->finalize     = gs_prefs_finalize;
         object_class->get_property = gs_prefs_get_property;
         object_class->set_property = gs_prefs_set_property;
@@ -404,7 +401,7 @@ key_changed_cb (GConfClient *client,
                     || value->type != GCONF_VALUE_LIST) {
                         return;
                 }
-                            
+
                 list = gconf_value_get_list (value);
 
                 if (list
@@ -564,9 +561,6 @@ gs_prefs_init (GSPrefs *prefs)
 
         prefs->priv->gconf_client      = gconf_client_get_default ();
 
-        prefs->verbose                 = FALSE;
-        prefs->debug                   = FALSE;
-
         prefs->idle_activation_enabled = TRUE;
         prefs->lock_enabled            = TRUE;
         prefs->logout_enabled          = FALSE;
@@ -577,7 +571,6 @@ gs_prefs_init (GSPrefs *prefs)
         prefs->logout_timeout          = 14400000;
         prefs->cycle                   = 600000;
 
-
         prefs->mode                    = GS_MODE_SINGLE;
 
         /* GConf setup */
@@ -618,7 +611,7 @@ gs_prefs_finalize (GObject *object)
 
         g_free (prefs->logout_command);
 
-        G_OBJECT_CLASS (parent_class)->finalize (object);
+        G_OBJECT_CLASS (gs_prefs_parent_class)->finalize (object);
 }
 
 GSPrefs *
