@@ -36,6 +36,7 @@
 #include <gtk/gtk.h>
 
 #include "gs-fade.h"
+#include "gs-debug.h"
 
 #ifdef HAVE_XF86VMODE_GAMMA
 # include <X11/extensions/xf86vmode.h>
@@ -55,18 +56,16 @@ test_fade (void)
         while (reps-- > 0) {
 
                 g_print ("fading out...");
-                gs_fade_now (fade);
+                gs_fade_sync (fade, 1000);
                 g_print ("done.\n");
 
-                if (delay)
-                        sleep (delay);
-
                 g_print ("fading in...");
                 gs_fade_reset (fade);
                 g_print ("done.\n");
 
-                if (delay)
+                if (delay) {
                         sleep (delay);
+                }
         }
 
         g_object_unref (fade);
@@ -83,9 +82,9 @@ main (int    argc,
         bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 # ifdef HAVE_BIND_TEXTDOMAIN_CODESET
         bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-# endif 
+# endif
         textdomain (GETTEXT_PACKAGE);
-#endif 
+#endif
 
         if (error) {
                 fprintf (stderr, "%s\n", error->message);
@@ -111,7 +110,11 @@ main (int    argc,
 # endif /* !HAVE_XF86VMODE_GAMMA */
         }
 
+        gs_debug_init (TRUE, FALSE);
+
         test_fade ();
 
+        gs_debug_shutdown ();
+
         return 0;
 }
